@@ -17,7 +17,7 @@
 %                                 July 2020                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -283,9 +283,9 @@ static TileInfo AssignBestTileLocation(AshlarInfo *ashlar_info,
       tile.y=0;
       return(tile);
     }
-  tile.x=(ssize_t) SSIZE_MAX;
-  tile.y=(ssize_t) SSIZE_MAX;
-  min_excess=(ssize_t) SSIZE_MAX;
+  tile.x=(ssize_t) MAGICK_SSIZE_MAX;
+  tile.y=(ssize_t) MAGICK_SSIZE_MAX;
+  min_excess=(ssize_t) MAGICK_SSIZE_MAX;
   node=ashlar_info->current;
   previous=(&ashlar_info->current);
   while ((width+node->x) <= ashlar_info->width)
@@ -448,7 +448,7 @@ static MagickBooleanType PackAshlarTiles(AshlarInfo *ashlar_info,
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -472,8 +472,8 @@ static MagickBooleanType PackAshlarTiles(AshlarInfo *ashlar_info,
         tiles[i].y=(ssize_t) tile_info.y;
         if (tile_info.previous == (NodeInfo **) NULL)
           {
-            tiles[i].x=(ssize_t) SSIZE_MAX;
-            tiles[i].y=(ssize_t) SSIZE_MAX;
+            tiles[i].x=(ssize_t) MAGICK_SSIZE_MAX;
+            tiles[i].y=(ssize_t) MAGICK_SSIZE_MAX;
           }
       }
   }
@@ -481,8 +481,8 @@ static MagickBooleanType PackAshlarTiles(AshlarInfo *ashlar_info,
   status=MagickTrue;
   for (i=0; i < (ssize_t) number_tiles; i++)
   {
-    tiles[i].order=(ssize_t) ((tiles[i].x != (ssize_t) SSIZE_MAX) ||
-      (tiles[i].y != (ssize_t) SSIZE_MAX) ? 1 : 0);
+    tiles[i].order=(ssize_t) ((tiles[i].x != (ssize_t) MAGICK_SSIZE_MAX) ||
+      (tiles[i].y != (ssize_t) MAGICK_SSIZE_MAX) ? 1 : 0);
     if (tiles[i].order == 0)
       status=MagickFalse;
   }
@@ -548,6 +548,9 @@ static MagickBooleanType WriteASHLARImage(const ImageInfo *image_info,
       geometry.height=(size_t) geometry.height/7;
       geometry.x=(ssize_t) pow((double) geometry.width,0.25);
       geometry.y=(ssize_t) pow((double) geometry.height,0.25);
+      value=GetImageOption(image_info,"label");
+      if (value != (const char *) NULL)
+        geometry.height+=1.8*geometry.y;
     }
   /*
     Initialize image tiles.
@@ -580,7 +583,7 @@ static MagickBooleanType WriteASHLARImage(const ImageInfo *image_info,
   value=GetImageOption(image_info,"ashlar:best-fit");
   for (i=20; i > 0; i--)
   {
-    register ssize_t
+    ssize_t
       j;
 
     n=0;
@@ -606,7 +609,7 @@ static MagickBooleanType WriteASHLARImage(const ImageInfo *image_info,
     ashlar_info.head.y=0;
     ashlar_info.head.next=(&ashlar_info.sentinal);
     ashlar_info.sentinal.x=(ssize_t) geometry.width;
-    ashlar_info.sentinal.y=(ssize_t) SSIZE_MAX;
+    ashlar_info.sentinal.y=(ssize_t) MAGICK_SSIZE_MAX;
     ashlar_info.sentinal.next=(NodeInfo *) NULL;
     status=PackAshlarTiles(&ashlar_info,tiles,(size_t) n);
     if (status != MagickFalse)
@@ -623,8 +626,8 @@ static MagickBooleanType WriteASHLARImage(const ImageInfo *image_info,
     Image
       *tile_image;
 
-    if ((tiles[i].x == (ssize_t) SSIZE_MAX) ||
-        (tiles[i].y == (ssize_t) SSIZE_MAX))
+    if ((tiles[i].x == (ssize_t) MAGICK_SSIZE_MAX) ||
+        (tiles[i].y == (ssize_t) MAGICK_SSIZE_MAX))
       continue;
     tile_image=ResizeImage(GetImageFromList(image,tiles[i].id),(size_t)
       (tiles[i].width-2*geometry.x),(size_t) (tiles[i].height-2*geometry.y),
@@ -640,7 +643,7 @@ static MagickBooleanType WriteASHLARImage(const ImageInfo *image_info,
           offset[MagickPathExtent];
 
         DrawInfo
-          *draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
+          *draw_info = CloneDrawInfo(image_info,(DrawInfo *) NULL);
 
         label=InterpretImageProperties((ImageInfo *) image_info,tile_image,
           value,exception);
