@@ -272,6 +272,8 @@ static int ReadSingleWEBPImage(Image *image,const uint8_t *stream,
     }
   else
     {
+      canvas_width=0;
+      canvas_height=0;
       x_offset=0;
       y_offset=0;
     }
@@ -749,18 +751,6 @@ static int WebPEncodeProgress(int percent,const WebPPicture* picture)
 }
 #endif
 
-#if !defined(MAGICKCORE_WEBPMUX_DELEGATE)
-static int WebPEncodeWriter(const unsigned char *stream,size_t length,
-  const WebPPicture *const picture)
-{
-  Image
-    *image;
-
-  image=(Image *) picture->custom_ptr;
-  return(length != 0 ? (WriteBlob(image,length,stream) == (ssize_t) length) : 1);
-}
-#endif
-
 static const char * WebPErrorCodeMessage(WebPEncodingError error_code)
 {
   switch (error_code)
@@ -1038,7 +1028,7 @@ static MagickBooleanType WriteWEBPImageProfile(Image *image,
       else
         if (mux_error == WEBP_MUX_OK)
           {
-            new_params.loop_count=MagickMin(image->iterations,65535);
+            new_params.loop_count=MagickMin((int) image->iterations,65535);
             mux_error=WebPMuxSetAnimationParams(mux, &new_params);
           }
     }
@@ -1079,6 +1069,17 @@ static MagickBooleanType WriteWEBPImageProfile(Image *image,
 }
 #endif
 
+static inline void SetBooleanOption(const ImageInfo *image_info,
+  const char *option,int *setting)
+{
+  const char
+    *value;
+
+  value=GetImageOption(image_info,option);
+  if (value != (char *) NULL)
+    *setting=(int) ParseCommandOption(MagickBooleanOptions,MagickFalse,value);
+}
+
 static inline void SetIntegerOption(const ImageInfo *image_info,
   const char *option,int *setting)
 {
@@ -1125,10 +1126,7 @@ static MagickBooleanType WriteWEBPImage(const ImageInfo *image_info,
     configure.quality=(float) image->quality;
   if (image->quality >= 100)
     configure.lossless=1;
-  value=GetImageOption(image_info,"webp:lossless");
-  if (value != (char *) NULL)
-    configure.lossless=(int) ParseCommandOption(MagickBooleanOptions,
-      MagickFalse,value);
+  SetBooleanOption(image_info,"webp:lossless",&configure.lossless);
   value=GetImageOption(image_info,"webp:image-hint");
   if (value != (char *) NULL)
     {
@@ -1143,10 +1141,7 @@ static MagickBooleanType WriteWEBPImage(const ImageInfo *image_info,
         configure.image_hint=WEBP_HINT_GRAPH;
 #endif
     }
-  value=GetImageOption(image_info,"webp:auto-filter");
-  if (value != (char *) NULL)
-    configure.autofilter=(int) ParseCommandOption(MagickBooleanOptions,
-      MagickFalse,value);
+  SetBooleanOption(image_info,"webp:auto-filter",&configure.autofilter);
   value=GetImageOption(image_info,"webp:target-psnr");
   if (value != (char *) NULL)
     configure.target_PSNR=(float) StringToDouble(value,(char **) NULL);
@@ -1173,19 +1168,17 @@ static MagickBooleanType WriteWEBPImage(const ImageInfo *image_info,
   SetIntegerOption(image_info,"webp:sns-strength",&configure.sns_strength);
   SetIntegerOption(image_info,"webp:target-size",&configure.target_size);
 #if WEBP_ENCODER_ABI_VERSION >= 0x0201
-  value=GetImageOption(image_info,"webp:emulate-jpeg-size");
-  if (value != (char *) NULL)
-    configure.emulate_jpeg_size=(int) ParseCommandOption(MagickBooleanOptions,
-      MagickFalse,value);
-  value=GetImageOption(image_info,"webp:low-memory");
-  if (value != (char *) NULL)
-    configure.low_memory=(int) ParseCommandOption(MagickBooleanOptions,
-      MagickFalse,value);
+  SetBooleanOption(image_info,"webp:emulate-jpeg-size",
+    &configure.emulate_jpeg_size);
+  SetBooleanOption(image_info,"webp:low-memory",&configure.low_memory);
   SetIntegerOption(image_info,"webp:thread-level",&configure.thread_level);
 #endif
+#if WEBP_ENCODER_ABI_VERSION >= 0x0209
+  SetBooleanOption(image_info,"webp:exact",&configure.exact);
+#endif
 #if WEBP_ENCODER_ABI_VERSION >= 0x020e
   SetIntegerOption(image_info,"webp:near-lossless",&configure.near_lossless);
-  SetIntegerOption(image_info,"webp:use-sharp-yuv",&configure.use_sharp_yuv);
+  SetBooleanOption(image_info,"webp:use-sharp-yuv",&configure.use_sharp_yuv);
 #endif
   if (WebPValidateConfig(&configure) == 0)
     ThrowWriterException(ResourceLimitError,"UnableToEncodeImageFile");
